# -*- coding: utf-8 -*-


import os
import time

import pytest
import shutil
import pandas

from academic_tracker.athr_srch_emails_and_reports import create_pubs_by_author_dict, create_project_reports_and_emails, create_project_report
from academic_tracker.athr_srch_emails_and_reports import create_summary_report, build_author_loop, create_collaborators_reports_and_emails
from academic_tracker.athr_srch_emails_and_reports import create_tabular_collaborator_report, create_collaborator_report, create_tabular_summary_report
from academic_tracker.athr_srch_emails_and_reports import create_tabular_project_report, replace_keywords
from academic_tracker.fileio import load_json, read_text_from_txt


TESTING_DIR = "test_dir"
@pytest.fixture(scope="module", autouse=True)
def test_email_dir():
    save_dir_name = TESTING_DIR
    os.mkdir(save_dir_name)
    time_to_wait = 10
    time_counter = 0
    
    while not os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_dir_name + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")



@pytest.fixture
def publication_dict():
    return load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))


def test_create_pubs_by_author_dict(publication_dict):
    expected = load_json(os.path.join("tests", "testing_files", "pubs_by_author_dict_truncated.json"))
    
    actual = create_pubs_by_author_dict(publication_dict)
    
    assert expected == actual



@pytest.fixture
def config_dict():
    return load_json(os.path.join("tests", "testing_files", "config_truncated.json"))


@pytest.fixture
def authors_by_project_dict():
    return load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_truncated.json"))


def test_create_project_reports_and_emails(publication_dict, config_dict, authors_by_project_dict):
    
    expected_emails = load_json(os.path.join("tests", "testing_files", "athr_project_emails.json"))
    del expected_emails["creation_date"]
    
    actual_emails = create_project_reports_and_emails(authors_by_project_dict, publication_dict, config_dict, TESTING_DIR)
    del actual_emails["creation_date"]
    
    dir_contents = os.listdir(TESTING_DIR)
    expected_num_of_project_reports = 13
    actual_num_of_project_reports = len(dir_contents)
    
    assert expected_emails == actual_emails
    assert actual_num_of_project_reports == expected_num_of_project_reports
    
    
@pytest.fixture
def config_dict_tabular():
    return load_json(os.path.join("tests", "testing_files", "config_tabular.json"))


@pytest.fixture
def authors_by_project_dict_tabular():
    return load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_tabular.json"))
    
    
def test_create_project_reports_and_emails_tabular(publication_dict, config_dict_tabular, authors_by_project_dict_tabular):
            
    expected_emails = load_json(os.path.join("tests", "testing_files", "athr_project_emails_tabular.json"))
    del expected_emails["creation_date"]
    ## attachment for xlsx files is a filepath and it differs between OS's.
    del expected_emails["emails"][2]["attachment"]
    
    actual_emails = create_project_reports_and_emails(authors_by_project_dict_tabular, publication_dict, config_dict_tabular, TESTING_DIR)
    del actual_emails["creation_date"]
    del actual_emails["emails"][2]["attachment"]
    
    dir_contents = os.listdir(TESTING_DIR)
    expected_num_of_project_reports = 13
    actual_num_of_project_reports = len(dir_contents)
    
    number_of_excel_files = len([name for name in dir_contents if ".xlsx" in name])
    
    assert expected_emails == actual_emails
    assert number_of_excel_files == 1
    assert "no_from_email_report.csv" in dir_contents
    assert actual_num_of_project_reports == expected_num_of_project_reports
        



def test_create_project_report(publication_dict, config_dict, authors_by_project_dict):
    
    template_string = "<author_loop><author_first> <author_last>:\n<pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>"
    author_first = "Kelly"
    author_last = "Pennell"
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "project_report.txt"))
    
    actual_text = create_project_report(publication_dict, config_dict, authors_by_project_dict, "Core A Administrative Core", template_string, author_first, author_last)
    
    assert expected_text == actual_text



def test_create_summary_report(publication_dict, config_dict, authors_by_project_dict):
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_summary_report.txt"))
    
    actual_text = create_summary_report(publication_dict, config_dict, authors_by_project_dict)
    
    assert expected_text == actual_text



def test_build_author_loop(publication_dict, config_dict, authors_by_project_dict):
    
    template_string = read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_build_loop_template_string.txt"))
                         
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_build_author_loop.txt"))
    
    actual_text = build_author_loop(publication_dict, config_dict, authors_by_project_dict, "No from_email", template_string)
    
    assert expected_text == actual_text
    
    


def test_create_collaborators_reports_and_emails_tabular(publication_dict, config_dict):
    
    del config_dict["Authors"]["Ann Koempel"]
    
    config_dict["Authors"]["Anna Hoover"]["collaborator_report"] = {"from_email":"ptth222@uky.edu", 
                                                                    "email_body":"asdf",
                                                                    "email_subject":"asdf",
                                                                    }
    config_dict["Authors"]["Kelly Pennell"]["collaborator_report"] = {"from_email":"ptth222@uky.edu",
                                                                      "to_email":"ptth222@uky.edu",
                                                                      "email_body":"asdf",
                                                                      "email_subject":"asdf",
                                                                      "columns":{"Col1":"<pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations>"},
                                                                      "separator":"\t"
                                                                      }
    config_dict["Authors"]["Lindell Ormsbee"]["collaborator_report"] = {"from_email":"ptth222@uky.edu",
                                                                        "to_email":"ptth222@uky.edu",
                                                                        "email_body":"asdf",
                                                                        "email_subject":"asdf",
                                                                        "columns":{"Col1":"<pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations>"},
                                                                        "sort":["Col1"]
                                                                        }
    
    expected_emails = load_json(os.path.join("tests", "testing_files", "collaborator_emails_tabular.json"))
    del expected_emails["creation_date"]
    
    actual_emails = create_collaborators_reports_and_emails(publication_dict, config_dict, TESTING_DIR)
    del actual_emails["creation_date"]
    
    dir_contents = os.listdir(TESTING_DIR)
    expected_num_of_collaborator_reports = 3
    actual_num_of_collaborator_reports = len(dir_contents)
    
    assert expected_emails == actual_emails
    assert actual_num_of_collaborator_reports == expected_num_of_collaborator_reports
    


def test_create_collaborators_reports_and_emails(publication_dict, config_dict):
    
    config_dict["Authors"]["Ann Koempel"]["collaborator_report"] = {"template":"<pub_author_loop><pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations></pub_author_loop>",
                                                                    "from_email":"ptth222@uky.edu",
                                                                    "email_body":"asdf",
                                                                    "email_subject":"asdf"}
    
    config_dict["Authors"]["Anna Hoover"]["collaborator_report"] = {"template":"<pub_author_loop><pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations></pub_author_loop>",
                                                                    "from_email":"ptth222@uky.edu",
                                                                    "email_body":"asdf",
                                                                    "email_subject":"asdf",
                                                                    "filename":"name_test.txt"}
    
    config_dict["Authors"]["Kelly Pennell"]["collaborator_report"] = {"from_email":"ptth222@uky.edu",
                                                                      "to_email":"ptth222@uky.edu",
                                                                      "email_body":"asdf",
                                                                      "email_subject":"asdf",
                                                                      "columns":{"Col1":"<pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations>"},
                                                                      "file_format":"xlsx"}
    
    config_dict["Authors"]["Lindell Ormsbee"]["collaborator_report"] = {"from_email":"ptth222@uky.edu",
                                                                        "to_email":"ptth222@uky.edu",
                                                                        "email_body":"asdf",
                                                                        "email_subject":"asdf",
                                                                        "columns":{"Col1":"<pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations>"},
                                                                        "sort":["Col1"],
                                                                        "filename":"name_test.csv"}
    
    expected_emails = load_json(os.path.join("tests", "testing_files", "collaborator_emails.json"))
    del expected_emails["creation_date"]
    del expected_emails["emails"][2]["attachment"]
    
    actual_emails = create_collaborators_reports_and_emails(publication_dict, config_dict, TESTING_DIR)
    del actual_emails["creation_date"]
    del actual_emails["emails"][2]["attachment"]
    
    dir_contents = os.listdir(TESTING_DIR)
    expected_num_of_collaborator_reports = 4
    actual_num_of_collaborator_reports = len(dir_contents)
    
    number_of_excel_files = len([name for name in dir_contents if ".xlsx" in name])
    
    assert expected_emails == actual_emails
    assert "name_test.txt" in dir_contents
    assert "name_test.csv" in dir_contents
    assert number_of_excel_files == 1
    assert actual_num_of_collaborator_reports == expected_num_of_collaborator_reports
    
    


@pytest.mark.parametrize("config_dict_collab, filename, file_format", [
        
        ({"Authors":{"Anna Hoover":{"collaborator_report":{"columns":{"Col1":"<pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations>", "Col2":"asdf"},
                                                           "separator":"\t",
                                                           "sort":["Col1"],
                                                           "column_order":["Col2", "Col1"]}}}}, "test_name.csv", "csv"),
        ({"Authors":{"Anna Hoover":{"collaborator_report":{"columns":{"Col1":"<pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations>", "Col2":"asdf"},
                                                           "column_order":["Col1", "Col2"]}}}}, "test_name.csv", "csv"),
        ])   

def test_create_tabular_collaborator_report(publication_dict, config_dict_collab, filename, file_format):
    
    if "separator" in config_dict_collab["Authors"]["Anna Hoover"]["collaborator_report"]:
        sep = config_dict_collab["Authors"]["Anna Hoover"]["collaborator_report"]["separator"]
    else:
        sep = ","
        
    if "sort" in config_dict_collab["Authors"]["Anna Hoover"]["collaborator_report"]:
        column_one = 'Dawn, Brewer, D, University of Kentucky Department of Dietetics and Human Nutrition.'
    else:
        column_one = 'W Jay, Christian, WJ, University of Kentucky College of Public Health.'
    
    create_tabular_collaborator_report(publication_dict, config_dict_collab, "Anna Hoover", ["32095784"], filename, file_format, TESTING_DIR)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert filename in dir_contents
    
    df = pandas.read_csv(os.path.join(TESTING_DIR, filename), sep=sep)
    report_text = read_text_from_txt(os.path.join(TESTING_DIR, filename))
    assert df.to_csv(sep=sep, index=False, lineterminator="\n") == report_text
    assert list(df.columns) == config_dict_collab["Authors"]["Anna Hoover"]["collaborator_report"]["column_order"]
    assert df.iloc[1].loc["Col1"] == column_one
    assert df.iloc[0].loc["Col2"] == "asdf"
    
    

def test_create_tabular_collaborator_report_defaults(publication_dict):
    
    config_dict = {"Authors":{"Anna Hoover":{"collaborator_report":{}}}}
    
    create_tabular_collaborator_report(publication_dict, config_dict, "Anna Hoover", ["32095784"], "test_name.csv", "csv", TESTING_DIR)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert "test_name.csv" in dir_contents
    
    df = pandas.read_csv(os.path.join(TESTING_DIR, "test_name.csv"), sep=",")
    report_text = read_text_from_txt(os.path.join(TESTING_DIR, "test_name.csv"))
    assert df.to_csv(sep=",", index=False, lineterminator="\n") == report_text
    assert list(df.columns) == ["Name", "Affiliations"]
    assert df.iloc[0].loc["Name"] == 'Brewer, Dawn'
    assert df.iloc[0].loc["Affiliations"] == 'University of Kentucky Department of Dietetics and Human Nutrition.'
    

def test_create_tabular_collaborator_report_excel(publication_dict):
    
    config_dict = {"Authors":{"Anna Hoover":{"collaborator_report":{}}}}
    
    create_tabular_collaborator_report(publication_dict, config_dict, "Anna Hoover", ["32095784"], "test_name.csv", "xlsx", TESTING_DIR)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert "test_name.csv.xlsx" in dir_contents
    
    df = pandas.read_excel(os.path.join(TESTING_DIR, "test_name.csv.xlsx"))
    assert list(df.columns) == ["Name", "Affiliations"]
    assert df.iloc[0].loc["Name"] == 'Brewer, Dawn'
    assert df.iloc[0].loc["Affiliations"] == 'University of Kentucky Department of Dietetics and Human Nutrition.'
    
    
def test_create_tabular_collaborator_report_empty():
    
    config_dict = {"Authors":{"Anna Hoover":{"collaborator_report":{}}}}
    publication_dict = {"32095784":{"authors":[]}}
    
    report, filename = create_tabular_collaborator_report(publication_dict, config_dict, "Anna Hoover", ["32095784"], "test_name.csv", "xlsx", TESTING_DIR)
    
    assert report == ""
    
    
    
def test_create_collaborator_report(publication_dict):
    
    template = "<pub_author_loop><pub_author_first>, <pub_author_last>, <pub_author_initials>, <pub_author_affiliations></pub_author_loop>"
    
    expected_report = 'Annie, Koempel, A, University of Kentucky Department of Dietetics and Human Nutrition.W Jay, Christian, WJ, University of Kentucky College of Public Health.Kimberly I, Tumlin, KI, University of Kentucky College of Public Health.Kelly G, Pennell, KG, University of Kentucky College of Engineering.Steven, Evans, S, Kentucky Water Resources Research Institute.Malissa, McAlister, M, Kentucky Water Resources Research Institute.Lindell E, Ormsbee, LE, University of Kentucky College of Engineering.Dawn, Brewer, D, University of Kentucky Department of Dietetics and Human Nutrition.'
    
    actual_report = create_collaborator_report(publication_dict, template, "Anna Hoover", ["32095784"], "test_name.txt", TESTING_DIR)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert "test_name.txt" in dir_contents
    assert expected_report == actual_report
    
    



def test_create_tabular_summary_report_no_pub_keywords(publication_dict, config_dict, authors_by_project_dict):
    
    config_dict["summary_report"] = {"columns":{"Col1":"<project_name>", "Col2":"<author_first>"},
                                     "filename":"test_name.csv",
                                     "file_format":"csv",
                                     "separator":"\t",
                                     "sort":["Col2"],
                                     "column_order":["Col2", "Col1"]}
    
    create_tabular_summary_report(publication_dict, config_dict, authors_by_project_dict, TESTING_DIR)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert "test_name.csv" in dir_contents
    
    df = pandas.read_csv(os.path.join(TESTING_DIR, "test_name.csv"), sep="\t")
    report_text = read_text_from_txt(os.path.join(TESTING_DIR, "test_name.csv"))
    assert df.to_csv(sep="\t", index=False, lineterminator="\n") == report_text
    assert list(df.columns) == config_dict["summary_report"]["column_order"]
    assert df.iloc[1].loc["Col1"] == "Project 1"
    assert df.iloc[0].loc["Col2"] == "Angela"
    
    

def test_create_tabular_summary_report_defaults(publication_dict, config_dict, authors_by_project_dict):
    
    config_dict["summary_report"] = {"columns":{"Col1":"<project_name>", "Col2":"<author_first>", "Col3":"<title>"}}
    
    create_tabular_summary_report(publication_dict, config_dict, authors_by_project_dict, TESTING_DIR)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert "summary_report.csv" in dir_contents
    
    df = pandas.read_csv(os.path.join(TESTING_DIR, "summary_report.csv"), sep=",")
    report_text = read_text_from_txt(os.path.join(TESTING_DIR, "summary_report.csv"))
    assert df.to_csv(sep=",", index=False, lineterminator="\n") == report_text
    assert list(df.columns) == list(config_dict["summary_report"]["columns"].keys())
    assert df.iloc[1].loc["Col1"] == 'Core A Administrative Core'
    assert df.iloc[0].loc["Col2"] == "Kelly"
    assert df.iloc[0].loc["Col3"] == 'Appalachian Environmental Health Literacy: Building Knowledge and Skills to Protect Health.'
    

def test_create_tabular_summary_report_excel(publication_dict, config_dict, authors_by_project_dict):
    
    config_dict["summary_report"] = {"columns":{"Col1":"<project_name>", "Col2":"<author_first>", "Col3":"<title>", "Col4":"<pub_author_first>"},
                                     "filename":"summary_report.csv",
                                     "file_format":"xlsx"}
    
    create_tabular_summary_report(publication_dict, config_dict, authors_by_project_dict, TESTING_DIR)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert "summary_report.csv.xlsx" in dir_contents
    
    df = pandas.read_excel(os.path.join(TESTING_DIR, "summary_report.csv.xlsx"))
    assert list(df.columns) == list(config_dict["summary_report"]["columns"].keys())
    assert df.iloc[1].loc["Col1"] == 'Core A Administrative Core'
    assert df.iloc[0].loc["Col2"] == "Kelly"
    assert df.iloc[0].loc["Col3"] == 'Appalachian Environmental Health Literacy: Building Knowledge and Skills to Protect Health.'
    assert df.iloc[0].loc["Col4"] == "Anna G"
    
    




@pytest.fixture
def pubs_by_author_dict():
    return load_json(os.path.join("tests", "testing_files", "pubs_by_author_dict_truncated.json"))

def test_create_tabular_project_report_no_pub_keywords(publication_dict, config_dict, authors_by_project_dict, pubs_by_author_dict):
    
    report_attributes = {"columns":{"Col1":"<project_name>", "Col2":"<author_first>"},
                         "filename":"test_name.csv",
                         "file_format":"csv",
                         "separator":"\t",
                         "sort":["Col2"],
                         "column_order":["Col2", "Col1"]}
    
    project_name = "Project 1"
    
    filename = "project_report.csv"
    
    create_tabular_project_report(publication_dict, config_dict, authors_by_project_dict, pubs_by_author_dict, project_name, report_attributes, TESTING_DIR, filename)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert filename in dir_contents
    
    df = pandas.read_csv(os.path.join(TESTING_DIR, filename), sep="\t")
    report_text = read_text_from_txt(os.path.join(TESTING_DIR, filename))
    assert df.to_csv(sep="\t", index=False, lineterminator="\n") == report_text
    assert list(df.columns) == report_attributes["column_order"]
    assert df.iloc[1].loc["Col1"] == "Project 1"
    assert df.iloc[0].loc["Col2"] == "Angela"
    
    

def test_create_tabular_project_report_defaults(publication_dict, config_dict, authors_by_project_dict, pubs_by_author_dict):
    
    report_attributes = {"columns":{"Col1":"<project_name>", "Col2":"<author_first>", "Col3":"<title>"}}
    
    project_name = "Project 1"
    
    filename = "project_report.csv"
    
    create_tabular_project_report(publication_dict, config_dict, authors_by_project_dict, pubs_by_author_dict, project_name, report_attributes, TESTING_DIR, filename)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert "project_report.csv" in dir_contents
    
    df = pandas.read_csv(os.path.join(TESTING_DIR, "project_report.csv"), sep=",")
    report_text = read_text_from_txt(os.path.join(TESTING_DIR, "project_report.csv"))
    assert df.to_csv(sep=",", index=False, lineterminator="\n") == report_text
    assert list(df.columns) == list(report_attributes["columns"].keys())
    assert df.iloc[1].loc["Col1"] == 'Project 1'
    assert df.iloc[0].loc["Col2"] == "Kelly"
    assert df.iloc[0].loc["Col3"] == 'Appalachian Environmental Health Literacy: Building Knowledge and Skills to Protect Health.'
    

def test_create_tabular_project_report_excel(publication_dict, config_dict, authors_by_project_dict, pubs_by_author_dict):
    
    report_attributes = {"columns":{"Col1":"<project_name>", "Col2":"<author_first>", "Col3":"<title>", "Col4":"<pub_author_first>"},
                         "filename":"project_report.csv",
                         "file_format":"xlsx"}
    
    project_name = "Project 1"
    
    filename = "project_report.csv"
    
    create_tabular_project_report(publication_dict, config_dict, authors_by_project_dict, pubs_by_author_dict, project_name, report_attributes, TESTING_DIR, filename)
    
    dir_contents = os.listdir(TESTING_DIR)
    
    assert "project_report.csv.xlsx" in dir_contents
    
    df = pandas.read_excel(os.path.join(TESTING_DIR, "project_report.csv.xlsx"))
    assert list(df.columns) == list(report_attributes["columns"].keys())
    assert df.iloc[1].loc["Col1"] == 'Project 1'
    assert df.iloc[0].loc["Col2"] == "Kelly"
    assert df.iloc[0].loc["Col3"] == 'Appalachian Environmental Health Literacy: Building Knowledge and Skills to Protect Health.'
    assert df.iloc[0].loc["Col4"] == "Anna G"
    
    


def test_replace_keywords(publication_dict, config_dict):
    
    template = {"Col1":"<project_name>", "Col2":"<author_first>", "Col3":"<title>", "Col4":"<pub_author_first>"}
    
    expected_template = {"Col1":"Project 1", "Col2":"Anna", "Col3":"Appalachian Environmental Health Literacy: Building Knowledge and Skills to Protect Health.", "Col4":"Travis"}
    
    pub_author = {"firstname":"Travis",
                  "lastname":"Thompson",
                  "initials":"P.T.T.",
                  "affiliation":"University of Kentucky"}
    
    actual_template = replace_keywords(template, publication_dict, config_dict, project_name="Project 1", author="Anna Hoover", pub="32095784", pub_author=pub_author)
    
    assert expected_template == actual_template
    

def test_replace_keywords_more_keywords(publication_dict, config_dict):
    
    template = {"Col1":"<first_author>", "Col2":"<last_author>", "Col3":"<authors>", "Col4":"<grants>", "Col5":"<publication_year>"}
    
    expected_template = {"Col1":'Hoover, Anna G', "Col2":'Brewer, Dawn', "Col3":'Anna G Hoover, Annie Koempel, W Jay Christian, Kimberly I Tumlin, Kelly G Pennell, Steven Evans, Malissa McAlister, Lindell E Ormsbee, Dawn Brewer', "Col4":'G08 LM013185, P30 ES026529, P42 ES007380, R01 ES032396', "Col5":"2020"}
    
    pub_author = {"firstname":"Travis",
                  "lastname":"Thompson",
                  "initials":"P.T.T.",
                  "affiliation":"University of Kentucky"}
    
    actual_template = replace_keywords(template, publication_dict, config_dict, project_name="Project 1", author="Anna Hoover", pub="32095784", pub_author=pub_author)
    
    assert expected_template == actual_template
    
    
    
def test_replace_keywords_no_change(publication_dict, config_dict):
    
    template = {"Col1":"<project_name>", "Col2":"<author_first>", "Col3":"<title>", "Col4":"<pub_author_first>"}
    
    expected_template = template
    
    actual_template = replace_keywords(template, publication_dict, config_dict)
    
    assert expected_template == actual_template
    
    



@pytest.fixture(autouse=True)
def test_dir_reset(request):
    def clean_test_dir():
        if os.path.exists(TESTING_DIR):
            for test_file in os.listdir(TESTING_DIR):
                os.remove(os.path.join(TESTING_DIR, test_file))
    request.addfinalizer(clean_test_dir)


@pytest.fixture(scope="module", autouse=True)
def cleanup(request):
    """Cleanup a testing directory once we are finished."""
    def remove_test_dir():
        if os.path.exists(TESTING_DIR):
            shutil.rmtree(TESTING_DIR)
    request.addfinalizer(remove_test_dir)




# -*- coding: utf-8 -*-

import pytest
import xml.etree.ElementTree as ET
import os
import pymed
from academic_tracker.helper_functions import modify_pub_dict_for_saving
from academic_tracker.fileio import load_json


@pytest.fixture
def authors_json_file():
    return load_json(os.path.join("tests", "testing_files", "authors.json"))

@pytest.fixture
def pub_with_no_matching_author():
    xml_path = os.path.join("tests", "testing_files", "no_author.xml")
    tree = ET.parse(xml_path)
    return pymed.article.PubMedArticle(xml_element=tree.getroot())


@pytest.fixture
def pub_with_matching_author():
    xml_path = os.path.join("tests", "testing_files", "has_author.xml")
    tree = ET.parse(xml_path)
    return pymed.article.PubMedArticle(xml_element=tree.getroot())


@pytest.fixture
def author_info():
    cutoff_year = 2019
    author_first_name = "Andrew"
    author_last_name = "Morris"
    author_affiliations = ["kentucky"]
    author_id = "Andrew Morris"
    
    return cutoff_year, author_first_name, author_last_name, author_affiliations, author_id

@pytest.fixture
def pub_with_grants():
    xml_path = os.path.join("tests", "testing_files", "has_pubmed_grants.xml")
    tree = ET.parse(xml_path)
    return pymed.article.PubMedArticle(xml_element=tree.getroot())

@pytest.fixture
def publication_dict(pub_with_grants, pub_with_matching_author):
    publication_dict = {}
    
    pub_dict = modify_pub_dict_for_saving(pub_with_grants)
    publication_dict["https://doi.org/" + pub_dict["doi"]] = pub_dict
    
    pub_dict = modify_pub_dict_for_saving(pub_with_matching_author)
    publication_dict["https://doi.org/" + pub_dict["doi"]] = pub_dict
    
    return publication_dict


@pytest.fixture
def pubs_by_author_dict():
    return {"Andrew Morris":{"34352431":set(), "33830777":set(["P42 ES007380"])}}

@pytest.fixture
def authors_dict():
    return {'Andrew Morris': {'email': 'a.j.morris@uky.edu',
                                  'first_name': 'Andrew',
                                  'last_name': 'Morris',
                                  'ORCID':'0000-0003-1910-4865',
                                  'scholar_id': '-j7fxnEAAAAJ',
                                  'pubmed_name_search': 'Andrew Morris',
                                  'affiliations': ['kentucky'],
                                  'cutoff_year': 2019,
                                  'grants': ['P42ES007380', 'P42 ES007380']}}

@pytest.fixture
def email_messages():
    return {'creation_date': '2021-10-20 21:50',
             'emails': [{'body': 'Hey Andrew,\n\nThese are the publications I was able to find on PubMed. Are any missing?\n\nMottaleb MA, Ding QX, Pennell KG, Haynes EN, Morris AJ. 2021. Direct injection analysis of per and polyfluoroalkyl substances in surface and drinking water by sample filtration and liquid chromatography-tandem mass spectrometry.. Journal of chromatography. A. 10.1016/j.chroma.2021.462426 PMID:34352431\n\nCited Grants:\nNone\n\n\nDeng P, Valentino T, Flythe MD, Moseley HNB, Leachman JR, Morris AJ, Hennig B. 2021. Untargeted Stable Isotope Probing of the Gut Microbiota Metabolome Using . Journal of proteome research. 10.1021/acs.jproteome.1c00124 PMID:33830777\n\nCited Grants:\nP42 ES007380\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu',
               'subject': 'New PubMed Publications',
               'from': 'ptth222@uky.edu',
               'to': 'a.j.morris@uky.edu',
               'cc': 'ptth222@uky.edu,ptth222@gmail.com',
               "attachment": "attachment content",
               "attachment_filename": "attachment_name.txt",
               'author': 'Andrew Morris'}]}
        

@pytest.fixture
def passing_config():
    return {"project_descriptions": {
                "project 1":{
                  "affiliations": [
                    "kentucky"
                  ],
                  "cutoff_year": 2019,
                  "project_report":{
                          "email_subject": "New PubMed Publications",
                          "email_body": "Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu",
                          "template": "Hey <author_first>,\n\nThese are the publications I was able to find on PubMed. Are any missing?\n\n<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>",
                          "from_email": "ptth222@uky.edu",
                          "cc_email": [],},
                  "collaborator_report":{},
                  "authors": ["Andrew Morris", "Hunter Moseley", "Isabel Escobar"],
                  "grants": [
                    "P42ES007380",
                    "P42 ES007380"
                  ]
                },
                "project 2":{
                  "affiliations": [
                    "kentucky"
                  ],
                  "cutoff_year": 2019,
                  "project_report":{
                          "email_subject": "New PubMed Publications",
                          "email_body": "Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu",
                          "template": "<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>",
                          "from_email": "ptth222@uky.edu",
                          "cc_email": [],
                          "to_email": ["ptth222@uky.edu"],},
                  "grants": [
                    "P42ES007380",
                    "P42 ES007380"
                  ]
                },
                "project 3":{
                  "affiliations": [
                    "kentucky"
                  ],
                  "cutoff_year": 2018,
                  "project_report":{
                          "email_subject": "New PubMed Publications",
                          "email_body": "Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu",
                          "template": "<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>",
                          "from_email": "ptth222@uky.edu",
                          "cc_email": [],
                          "to_email": ["ptth222@uky.edu"],},
                  "grants": [
                    "P42ES007380",
                    "P42 ES007380"
                  ]
                }
                },
            "ORCID_search":{
                    "ORCID_key":"asdf",
                    "ORCID_secret":"qwer"},
            "PubMed_search":{
                    "PubMed_email":"ptth222@uky.edu"},
            "Crossref_search":{
                    "mailto_email":"ptth222@uky.edu"},
            "summary_report":{
                    "email_subject": "New PubMed Publications",
                    "email_body": "Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu",
                    "template": "<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>",
                    "from_email": "ptth222@uky.edu",
                    "cc_email": [],
                    "to_email": ["ptth222@uky.edu"],},
            "Authors": {
                    "Andrew Morris": {
                          "ORCID": "0000-0003-1910-4865",
                          "affiliations": [
                            "kentucky"
                          ],
                          "cutoff_year": 2020,
                          "email": "a.j.morris@uky.edu",
                          "first_name": "Andrew",
                          "last_name": "Morris",
                          "pubmed_name_search": "Andrew Morris",
                          "scholar_id": "-j7fxnEAAAAJ"},
                    "Hunter Moseley": {
                          "ORCID": "0000-0003-3995-5368",
                          "affiliations": [
                            "kentucky"
                          ],
                          "cutoff_year": 2020,
                          "email": "hunter.moseley@gmail.com",
                          "first_name": "Hunter",
                          "last_name": "Moseley",
                          "pubmed_name_search": "Hunter Moseley",
                          "scholar_id": "ctE_FZMAAAAJ"},},
            }
            

@pytest.fixture
def authors_by_project_dict():
    return {'project 1': {
              'Andrew Morris': {'ORCID': '0000-0003-1910-4865',
                   'affiliations': ['kentucky'],
                   'cutoff_year': 2020,
                   'email': 'a.j.morris@uky.edu',
                   'first_name': 'Andrew',
                   'last_name': 'Morris',
                   'pubmed_name_search': 'Andrew Morris',
                   'scholar_id': '-j7fxnEAAAAJ',
                   'project_report': {'email_subject': 'New PubMed Publications',
                    'email_body': 'Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu',
                    'template': 'Hey <author_first>,\n\nThese are the publications I was able to find on PubMed. Are any missing?\n\n<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>',
                    'from_email': 'ptth222@uky.edu',
                    'cc_email': []},
                   'collaborator_report': {},
                   'authors': ['Andrew Morris', 'Hunter Moseley', 'Isabel Escobar'],
                   'grants': ['P42ES007380', 'P42 ES007380']},
              'Hunter Moseley': {'ORCID': '0000-0003-3995-5368',
                   'affiliations': ['kentucky'],
                   'cutoff_year': 2020,
                   'email': 'hunter.moseley@gmail.com',
                   'first_name': 'Hunter',
                   'last_name': 'Moseley',
                   'pubmed_name_search': 'Hunter Moseley',
                   'scholar_id': 'ctE_FZMAAAAJ',
                   'project_report': {'email_subject': 'New PubMed Publications',
                    'email_body': 'Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu',
                    'template': 'Hey <author_first>,\n\nThese are the publications I was able to find on PubMed. Are any missing?\n\n<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>',
                    'from_email': 'ptth222@uky.edu',
                    'cc_email': []},
                   'collaborator_report': {},
                   'authors': ['Andrew Morris', 'Hunter Moseley', 'Isabel Escobar'],
                   'grants': ['P42ES007380', 'P42 ES007380']}},
            'project 2': {
              'Andrew Morris': {'ORCID': '0000-0003-1910-4865',
                   'affiliations': ['kentucky'],
                   'cutoff_year': 2020,
                   'email': 'a.j.morris@uky.edu',
                   'first_name': 'Andrew',
                   'last_name': 'Morris',
                   'pubmed_name_search': 'Andrew Morris',
                   'scholar_id': '-j7fxnEAAAAJ',
                   'project_report': {'email_subject': 'New PubMed Publications',
                    'email_body': 'Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu',
                    'template': '<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>',
                    'from_email': 'ptth222@uky.edu',
                    'cc_email': [],
                    'to_email': ['ptth222@uky.edu']},
                   'grants': ['P42ES007380', 'P42 ES007380']},
              'Hunter Moseley': {'ORCID': '0000-0003-3995-5368',
                   'affiliations': ['kentucky'],
                   'cutoff_year': 2020,
                   'email': 'hunter.moseley@gmail.com',
                   'first_name': 'Hunter',
                   'last_name': 'Moseley',
                   'pubmed_name_search': 'Hunter Moseley',
                   'scholar_id': 'ctE_FZMAAAAJ',
                   'project_report': {'email_subject': 'New PubMed Publications',
                    'email_body': 'Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu',
                    'template': '<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>',
                    'from_email': 'ptth222@uky.edu',
                    'cc_email': [],
                    'to_email': ['ptth222@uky.edu']},
                   'grants': ['P42ES007380', 'P42 ES007380']}},
            'project 3': {
              'Andrew Morris': {'ORCID': '0000-0003-1910-4865',
                   'affiliations': ['kentucky'],
                   'cutoff_year': 2020,
                   'email': 'a.j.morris@uky.edu',
                   'first_name': 'Andrew',
                   'last_name': 'Morris',
                   'pubmed_name_search': 'Andrew Morris',
                   'scholar_id': '-j7fxnEAAAAJ',
                   'project_report': {'email_subject': 'New PubMed Publications',
                    'email_body': 'Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu',
                    'template': '<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>',
                    'from_email': 'ptth222@uky.edu',
                    'cc_email': [],
                    'to_email': ['ptth222@uky.edu']},
                   'grants': ['P42ES007380', 'P42 ES007380']},
              'Hunter Moseley': {'ORCID': '0000-0003-3995-5368',
                   'affiliations': ['kentucky'],
                   'cutoff_year': 2020,
                   'email': 'hunter.moseley@gmail.com',
                   'first_name': 'Hunter',
                   'last_name': 'Moseley',
                   'pubmed_name_search': 'Hunter Moseley',
                   'scholar_id': 'ctE_FZMAAAAJ',
                   'project_report': {'email_subject': 'New PubMed Publications',
                    'email_body': 'Attached is the report for publications found.\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu',
                    'template': '<author_loop><pub_loop>\t<title> <authors> <grants>\n</pub_loop></author_loop>',
                    'from_email': 'ptth222@uky.edu',
                    'cc_email': [],
                    'to_email': ['ptth222@uky.edu']},
                   'grants': ['P42ES007380', 'P42 ES007380']}}}
        
        


# -*- coding: utf-8 -*-


import os
import time

import pytest
import shutil
import pandas

from academic_tracker.ref_srch_emails_and_reports import convert_tokenized_authors_to_str, create_report_from_template, create_tokenization_report
from academic_tracker.ref_srch_emails_and_reports import create_tabular_report, replace_keywords
from academic_tracker.fileio import load_json, read_text_from_txt


TESTING_DIR = "test_dir"
@pytest.fixture(scope="module", autouse=True)
def test_email_dir():
    save_dir_name = TESTING_DIR
    os.mkdir(save_dir_name)
    time_to_wait = 10
    time_counter = 0
    
    while not os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_dir_name + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")


@pytest.fixture
def tokenized_citations():
    return load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))


def test_convert_tokenized_authors_to_str(tokenized_citations):
    
    expected_string = "Powell C, Moseley H, Travis Thompson"
    
    authors = tokenized_citations[0]["authors"]
    authors.append({"last":"Thompson", "first":"Travis"})
    actual_string = convert_tokenized_authors_to_str(authors)
    
    assert expected_string == actual_string


def test_convert_tokenized_authors_to_str_no_output():
    
    expected_string = str(None)
    
    actual_string = convert_tokenized_authors_to_str([])
    
    assert expected_string == actual_string



@pytest.fixture
def publication_dict():
    return load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))


def test_create_report_from_template_no_comparison(publication_dict, tokenized_citations):
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_test1.txt"))
    
    template_string = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_template_string.txt"))
    actual_text = create_report_from_template(publication_dict, [], tokenized_citations, template_string)
    
    assert expected_text == actual_text
    
    
def test_create_report_from_template_no_reference(publication_dict, tokenized_citations):
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_test2.txt"))
    
    template_string = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_template_string.txt"))
    for citation in tokenized_citations:
        citation["reference_line"] = ""
    actual_text = create_report_from_template(publication_dict, [True, False], tokenized_citations, template_string)
    
    assert expected_text == actual_text



@pytest.fixture
def config_dict():
    config_dict = {"summary_report":{"columns":{'Authors': '<authors>',
                                                'Grants': '<grants>',
                                                'Abstract': '<abstract>',
                                                'Conclusions': '<conclusions>',
                                                'Copyrights': '<copyrights>',
                                                'DOI': '<DOI>',
                                                'Journal': '<journal>',
                                                'Keywords': '<keywords>',
                                                'Methods': '<methods>',
                                                'PMID': '<PMID>',
                                                'Results': '<results>',
                                                'Title': '<title>',
                                                'PMCID': '<PMCID>',
                                                'Publication Year': '<publication_year>',
                                                'Publication Month': '<publication_month>',
                                                'Publication Day': '<publication_day>',
                                                'Tok Title': '<tok_title>',
                                                'Tok DOI': '<tok_DOI>',
                                                'Tok PMID': '<tok_PMID>',
                                                'Tok Authors': '<tok_authors>',
                                                'Ref Line': '<ref_line>',
                                                'Comparison': '<is_in_comparison_file>',
                                                'First Author': '<first_author>',
                                                'Last Author': '<last_author>',
                                                'Pub_Authors': '<pub_author_last>, <pub_author_first> <pub_author_initials> <pub_author_affiliations>'},
                                     "column_order":['Authors', 'Grants', 'Abstract', 'Conclusions', 'Copyrights', 'DOI', 'Journal', 'Keywords', 'Methods',
                                                     'PMID', 'Results', 'Title', 'PMCID', 'Publication Year', 'Publication Month', 'Publication Day',
                                                     'Tok Title', 'Tok DOI', 'Tok PMID', 'Tok Authors', 'Ref Line', 'Comparison', 'First Author', 
                                                     'Last Author', 'Pub_Authors'],
                                     "sort":["Authors"],
                                     "file_format":"csv",
                                     "filename":"test_name.csv",
                                     "separator":"\t"}}
    
    return config_dict


def test_create_tabular_report_no_comparison(publication_dict, tokenized_citations, config_dict):
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_tabular1.csv"))
    
    report, filename = create_tabular_report(publication_dict, config_dict, [], tokenized_citations, TESTING_DIR)
    
    actual_text = read_text_from_txt(os.path.join(TESTING_DIR, filename))
    
    assert expected_text == actual_text
    

def test_create_tabular_report_no_reference(publication_dict, tokenized_citations, config_dict):
    
    for citation in tokenized_citations:
        citation["reference_line"] = ""
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_tabular2.csv"))
    
    report, filename = create_tabular_report(publication_dict, config_dict, [True, False], tokenized_citations, TESTING_DIR)
    
    actual_text = read_text_from_txt(os.path.join(TESTING_DIR, filename))
    
    assert expected_text == actual_text
    

def test_create_tabular_report_defaults(publication_dict, tokenized_citations, config_dict):
    
    config_dict["summary_report"] = {"columns":config_dict["summary_report"]["columns"]}
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_tabular3.csv"))
    
    report, filename = create_tabular_report(publication_dict, config_dict, [], tokenized_citations, TESTING_DIR)
    
    actual_text = read_text_from_txt(os.path.join(TESTING_DIR, filename))
    
    assert expected_text == actual_text
    
    
def test_create_tabular_report_excel(publication_dict, tokenized_citations, config_dict):
    
    config_dict["summary_report"] = {"columns":config_dict["summary_report"]["columns"]}
    config_dict["summary_report"]["file_format"] = ".xlsx"
    config_dict["summary_report"]["filename"] = "test_name.csv"
    
    expected_text = pandas.read_excel(os.path.join("tests", "testing_files", "ref_srch_report_tabular4.xlsx"))
    
    report, filename = create_tabular_report(publication_dict, config_dict, [], tokenized_citations, TESTING_DIR)
    
    actual_text = pandas.read_excel(os.path.join(TESTING_DIR, filename))
    
    assert expected_text.to_csv() == actual_text.to_csv()
    
    


def test_replace_keywords1(publication_dict, config_dict, tokenized_citations):
    
    template = config_dict["summary_report"]["columns"]
    
    expected_template = {'Authors': 'Huan Jin, Joshua M. Mitchell, Hunter N. B. Moseley',
                         'Grants': 'None Found',
                         'Abstract': 'None',
                         'Conclusions': 'None',
                         'Copyrights': 'None',
                         'DOI': '10.3390/metabo10090368',
                         'Journal': 'MDPI AG',
                         'Keywords': 'None',
                         'Methods': 'None',
                         'PMID': 'None',
                         'Results': 'None',
                         'Title': 'Atom Identifiers Generated by a Neighborhood-Specific Graph Coloring Method Enable Compound Harmonization across Metabolic Databases',
                         'PMCID': 'None',
                         'Publication Year': '2020',
                         'Publication Month': 'None',
                         'Publication Day': 'None',
                         'Tok Title': 'Atom Identifiers Generated by a Neighborhood-Specific Graph Coloring Method Enable Compound Harmonization across Metabolic Databases.',
                         'Tok DOI': '10.3390/metabo10090368',
                         'Tok PMID': 'None',
                         'Tok Authors': 'Jin H, Mitchell J, Moseley H',
                         'Ref Line': 'Jin H, Mitchell J, Moseley H.  Atom Identifiers Generated by a Neighborhood-Specific Graph Coloring Method Enable Compound Harmonization across Metabolic Databases. Metabolites. 2020 September; 10(9):368-. doi: 10.3390/metabo10090368.',
                         'Comparison': 'N/A',
                         'First Author': 'Jin, Huan',
                         'Last Author': 'Moseley, Hunter N. B.',
                         'Pub_Authors': '<pub_author_last>, <pub_author_first> <pub_author_initials> <pub_author_affiliations>'}
    
    actual_template = replace_keywords(template, publication_dict, 'https://doi.org/10.3390/metabo10090368', tokenized_citations[1], None, pub_author={})
    
    assert expected_template == actual_template
    
    
def test_replace_keywords2(publication_dict, config_dict, tokenized_citations):
    
    template = config_dict["summary_report"]["columns"]
    publication_dict['https://doi.org/10.3390/metabo10090368']["grants"] = ["asdf"]
    tokenized_citations[1]["reference_line"] = ""
    
    expected_template = {'Authors': 'Huan Jin, Joshua M. Mitchell, Hunter N. B. Moseley',
                         'Grants': 'asdf',
                         'Abstract': 'None',
                         'Conclusions': 'None',
                         'Copyrights': 'None',
                         'DOI': '10.3390/metabo10090368',
                         'Journal': 'MDPI AG',
                         'Keywords': 'None',
                         'Methods': 'None',
                         'PMID': 'None',
                         'Results': 'None',
                         'Title': 'Atom Identifiers Generated by a Neighborhood-Specific Graph Coloring Method Enable Compound Harmonization across Metabolic Databases',
                         'PMCID': 'None',
                         'Publication Year': '2020',
                         'Publication Month': 'None',
                         'Publication Day': 'None',
                         'Tok Title': 'Atom Identifiers Generated by a Neighborhood-Specific Graph Coloring Method Enable Compound Harmonization across Metabolic Databases.',
                         'Tok DOI': '10.3390/metabo10090368',
                         'Tok PMID': 'None',
                         'Tok Authors': 'Jin H, Mitchell J, Moseley H',
                         'Ref Line': 'N/A',
                         'Comparison': 'True',
                         'First Author': 'Jin, Huan',
                         'Last Author': 'Moseley, Hunter N. B.',
                         'Pub_Authors': 'Jin, Huan None None'}
    
    actual_template = replace_keywords(template, publication_dict, 'https://doi.org/10.3390/metabo10090368', tokenized_citations[1], True, publication_dict['https://doi.org/10.3390/metabo10090368']["authors"][0])
    
    assert expected_template == actual_template




def test_create_tokenization_report(tokenized_citations):
    
    tokenized_citations.append({"authors":[], "title":"", "DOI":"", "PMID":"", "reference_line":""})
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "tokenization_report.txt"))
    
    actual_text = create_tokenization_report(tokenized_citations)
    
    assert expected_text == actual_text



@pytest.fixture(scope="module", autouse=True)
def cleanup(request):
    """Cleanup a testing directory once we are finished."""
    def remove_test_dir():
        if os.path.exists(TESTING_DIR):
            shutil.rmtree(TESTING_DIR)
    request.addfinalizer(remove_test_dir)



# -*- coding: utf-8 -*-

import os

import pytest
import pymed
import pickle
import requests
import xml.etree.ElementTree as ET

from academic_tracker.ref_srch_webio import build_pub_dict_from_PMID, search_references_on_PubMed, search_references_on_Crossref
from academic_tracker.ref_srch_webio import parse_myncbi_citations, tokenize_reference_input
from academic_tracker.fileio import load_json, read_text_from_txt



@pytest.fixture(autouse=True)
def disable_network_calls(monkeypatch):
    def stunted_get():
        raise RuntimeError("Network access not allowed during testing!")
    monkeypatch.setattr(requests, "get", lambda *args, **kwargs: stunted_get())


## pymed's PubMed.query() actually returns an itertools.chain where the elements are actually the _getArticles method of PubMed, not the articles directly.
## Each call to the iterator does an http request. Here I simply put the articles in a list. 
@pytest.fixture
def pymed_query():
    with open(os.path.join("tests", "testing_files", "pymed_pubs.pkl"), "rb") as f:
        articles = pickle.load(f)
    return articles


@pytest.fixture
def expected_publication_dict():
    return {'https://doi.org/10.1016/j.chroma.2021.462426': {'pubmed_id': '34352431',
  'title': 'Direct injection analysis of per and polyfluoroalkyl substances in surface and drinking water by sample filtration and liquid chromatography-tandem mass spectrometry.',
  'abstract': 'We developed and validated a method for direct determination of per- and polyfluoroalkylated substances (PFASs) in environmental water samples without prior sample concentration. Samples are centrifuged and supernatants passed through an Acrodisc Filter (GXF/GHP 0.2\xa0\xa0um, 25\xa0\xa0mm diameter). After addition of ammonium acetate, samples are analyzed by UPLC-MS/MS using an AB Sciex 6500 plus Q-Trap mass spectrometer operated in negative multiple reaction-monitoring (MRM) mode. The instrument system incorporates a delay column between the pumps and autosampler to mitigate interference from background PFAS. The method monitors eight short-/long-chain PFAS which are identified by monitoring specific precursor product ion pairs and by their retention times and quantified using isotope mass-labeled internal standard based calibration plots. Average spiked recoveries (n\xa0=\xa08) of target analytes ranged from 84 to 110% with 4-9% relative standard deviation (RSD). The mean spiked recoveries (n\xa0=\xa08) of four surrogates were 94-106% with 3-8% RSD. For continuous calibration verification (CCV), average spiked recoveries (n\xa0=\xa08) for target analytes ranged from 88 to 114% with 4-11% RSD and for surrogates ranged from 104-112% with 3-11% RSD. The recoveries (n\xa0=\xa06) of matrix spike (MX), matrix spike duplicate (MXD), and field reagent blank (FRB) met our acceptance criteria. The limit of detection for the target analytes was between 0.007 and 0.04\xa0ng/mL. The method was used to measure PFAS in tap water and surface water.',
  'keywords': ['Acrodisc filtration',
   'Direct injection',
   'Drinking and surface water',
   'PFAS',
   'UPLC-MS/MS'],
  'journal': 'Journal of chromatography. A',
  'publication_date': {'year': 2021, 'month': 8, 'day': 6},
  'authors': [{'lastname': 'Mottaleb',
    'firstname': 'M Abdul',
    'initials': 'MA',
    'affiliation': 'Superfund Research Center, University of Kentucky, Lexington KY, 40506, United States; Center for Appalachian Research in Environmental Sciences, University of Kentucky, Lexington KY, 40506, United States; Division of Cardiovascular, Medicine, College of Medicine, University of Kentucky and Lexington VA Medical Center, Lexington, KY, 40536, United States. a.j.morris@uky.edu; Pressent address: Institute of Drug & Biotherapeutic Innovation, DRC, 1100 South Grand Blvd, Saint Louis University, Saint Louis, MO 63104 United States. Electronic address: m.a.mottaleb@uky.edu.'},
   {'lastname': 'Ding',
    'firstname': 'Qunxing X',
    'initials': 'QX',
    'affiliation': 'Department of Biology, College of Arts and Sciences, Kent State University, Kent, OH, 44242, United States. Electronic address: qding@kent.edu.'},
   {'lastname': 'Pennell',
    'firstname': 'Kelly G',
    'initials': 'KG',
    'affiliation': 'Superfund Research Center, University of Kentucky, Lexington KY, 40506, United States; Center for Appalachian Research in Environmental Sciences, University of Kentucky, Lexington KY, 40506, United States; Department of Civil Engineering, College of Engineering, University of Kentucky, Lexington KY, 40506, United States. Electronic address: kellypennell@uky.edu.',},
   {'lastname': 'Haynes',
    'firstname': 'Erin N',
    'initials': 'EN',
    'affiliation': 'Superfund Research Center, University of Kentucky, Lexington KY, 40506, United States; Center for Appalachian Research in Environmental Sciences, University of Kentucky, Lexington KY, 40506, United States; Department of Epidemiology, College of Public Health, University of Kentucky, Lexington KY, 40536, United States. Electronic address: erin.haynes@uky.edu.',},
   {'lastname': 'Morris',
    'firstname': 'Andrew J',
    'initials': 'AJ',
    'affiliation': 'Superfund Research Center, University of Kentucky, Lexington KY, 40506, United States; Center for Appalachian Research in Environmental Sciences, University of Kentucky, Lexington KY, 40506, United States; Division of Cardiovascular, Medicine, College of Medicine, University of Kentucky and Lexington VA Medical Center, Lexington, KY, 40536, United States. a.j.morris@uky.edu; Pressent address: Institute of Drug & Biotherapeutic Innovation, DRC, 1100 South Grand Blvd, Saint Louis University, Saint Louis, MO 63104 United States. Electronic address: a.j.morris@uky.edu.',
    'author_id': 'Andrew Morris'}],
  'methods': None,
  'conclusions': None,
  'results': None,
  'copyrights': 'Copyright © 2021. Published by Elsevier B.V.',
  'doi': '10.1016/j.chroma.2021.462426',
  'grants': ['P30 ES026529'],
  'PMCID': None}}


def test_build_pub_dict_from_PMID(pymed_query, mocker, expected_publication_dict):
    def mock_query(*args, **kwargs):
        return pymed_query
    mocker.patch("academic_tracker.ref_srch_webio.pymed.PubMed.query", mock_query)
    test_publication_dict = build_pub_dict_from_PMID(['34352431', '11111111'], "ptth222@uky.edu")
    assert test_publication_dict == expected_publication_dict



## For whatever reason I couldn't pickle a list of pymed articles like I did before, 
## so I saved them as a string. Here they are loaded back in and converted to an article.
@pytest.fixture
def ref_pymed_query():
    xml_strings = load_json(os.path.join("tests", "testing_files", "ref_srch_PubMed_pubs.json"))
    return [pymed.article.PubMedArticle(xml_element=ET.fromstring(string)) for string in xml_strings]


@pytest.fixture
def tokenized_citations():
    tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations.json"))
    return [tokenized_citations[i] for i in [6,8,63]]
    


def test_search_references_on_PubMed(tokenized_citations, ref_pymed_query, mocker):
    def mock_query(*args, **kwargs):
        return ref_pymed_query
    mocker.patch("academic_tracker.ref_srch_webio.pymed.PubMed.query", mock_query)
    
    expected_publication_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_publication_dict.json"))
    expected_citation_keys = load_json(os.path.join("tests", "testing_files", "ref_srch_keys_for_citations.json"))
    
    actual_publication_dict, actual_citation_keys = search_references_on_PubMed(tokenized_citations, "ptth222@uky.edu")
    
    assert expected_publication_dict == actual_publication_dict
    assert expected_citation_keys == actual_citation_keys
    



def test_search_references_on_Crossref(tokenized_citations, mocker):
    def query_generator():
        queries = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_queries.json"))
        for query in queries:
            yield query
            
    queries = query_generator()
        
    def mock_query(*args, **kwargs):
        return next(queries)
    mocker.patch("academic_tracker.ref_srch_webio.habanero.Crossref.works", mock_query)
       
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    expected_citation_keys = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_keys_for_citations.json"))
    
    actual_pub_dict, actual_citation_keys = search_references_on_Crossref(tokenized_citations, "ptth222@uky.edu")
        
    assert actual_pub_dict == expected_pub_dict
    assert actual_citation_keys == expected_citation_keys




def test_parse_myncbi_citations(mocker):
    def page_generator():
        pages = load_json(os.path.join("tests", "testing_files", "myncbi_webpages.json"))
        for page in pages:
            yield page
            
    pages = page_generator()
        
    def mock_query(*args, **kwargs):
        return next(pages)
    mocker.patch("academic_tracker.ref_srch_webio.webio.get_url_contents_as_str", mock_query)
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations.json"))
    
    actual_tokenized_citations = parse_myncbi_citations("asdf")
    
    assert expected_tokenized_citations == actual_tokenized_citations



## Test that the JSON read in works and finds and eliminates duplicates.
def test_tokenize_reference_input_JSON(capsys):
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json"))
    
    actual_tokenized_citations = tokenize_reference_input(os.path.join("tests", "testing_files", "tokenized_citations.json"), False)
    captured = capsys.readouterr()
    
    assert expected_tokenized_citations == actual_tokenized_citations
    ## There should be something about duplicates in stdout.
    assert len(captured.out) > 0



def test_tokenize_reference_input_html(mocker):
    def mock_query(*args, **kwargs):
        return read_text_from_txt(os.path.join("tests", "testing_files", "nsf_award_page.txt"))
    mocker.patch("academic_tracker.ref_srch_webio.webio.clean_tags_from_url", mock_query)
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_nsf_award_page.json"))
    
    actual_tokenized_citations = tokenize_reference_input("https://www.nsf.gov/awardsearch/showAward?AWD_ID=1419282", False)
    
    assert expected_tokenized_citations == actual_tokenized_citations



def test_tokenize_reference_input_no_html_content(capsys, mocker):
    def mock_query(*args, **kwargs):
        return read_text_from_txt(os.path.join("tests", "testing_files", "empty_file.txt"))
    mocker.patch("academic_tracker.ref_srch_webio.webio.clean_tags_from_url", mock_query)
    
    with pytest.raises(SystemExit):
        tokenize_reference_input("https://www.nsf.gov/awardsearch/showAward?AWD_ID=1419282", False)
    captured = capsys.readouterr()
    
    assert captured.out == "Nothing was read from the URL. Make sure the address is correct.\n"



def test_tokenize_reference_input_docx():
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_ref_test.json"))
    
    actual_tokenized_citations = tokenize_reference_input(os.path.join("tests", "testing_files", "reference_test.docx"), False)
    
    assert expected_tokenized_citations == actual_tokenized_citations



def test_tokenize_reference_input_txt():
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_ref_test.json"))
    
    actual_tokenized_citations = tokenize_reference_input(os.path.join("tests", "testing_files", "reference_test.txt"), False)
    
    assert expected_tokenized_citations == actual_tokenized_citations



def test_tokenize_reference_input_wrong_file_extension(capsys):
    
    with pytest.raises(SystemExit):
        tokenize_reference_input("asdf.asdf", False)
    captured = capsys.readouterr()
    
    assert captured.out == "Unknown file type for reference file.\n"



def test_tokenize_reference_input_empty_file(capsys):
    
    with pytest.raises(SystemExit):
        tokenize_reference_input(os.path.join("tests", "testing_files", "empty_file.txt"), False)
    captured = capsys.readouterr()
    
    assert captured.out == "Nothing was read from the reference file. Make sure the file is not empty or is a supported file type.\n"



def test_tokenize_reference_input_MEDLINE():
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_MEDLINE.json"))
    
    actual_tokenized_citations = tokenize_reference_input(os.path.join("tests", "testing_files", "medline.txt"), True)
    
    assert expected_tokenized_citations == actual_tokenized_citations



def test_tokenize_reference_input_no_references(capsys):
    
    with pytest.raises(SystemExit):
        tokenize_reference_input(os.path.join("tests", "testing_files", "testing_text.txt"), False)
    captured = capsys.readouterr()
    
    assert captured.out == "Warning: Could not tokenize any citations in provided reference. Check setup and formatting and try again.\n"







